# Building Rest Services

1. [Setting up](#1-setting-up)
2. [Inserting initial data set](#2-inserting-initial-data-set)
3. [Configure Mongoose](#3-configure-mongoose)
4. [Creating Rest Services](#4-creating-rest-services)
5. [How to enable CORS?](#5-how-to-enable-CORS)

## 1- Setting up

1- Copy lab4 as lab6
2- Delete routes folder. We don’t need this folder.
3- Start mongodb

Example:
```shell
mongod --dbpath d:\dev\db\MongoDB-data
```

4- Open a command prompt and switch to lab6/shopping-services-app folder.

5- Type “npm install mongoose --save”  to install mongoose.

## 2- Inserting initial data set

1- Create a new file named init.js under root of the project. This file will be used for inserting  initial data set into the database
2- Edit file content as shown below:

```javascript
var mongoose = require('mongoose');

var db = mongoose.connection;
db.on('error', console.error);
mongoose.connect('mongodb://localhost/shoppingdb');

var productSchema = mongoose.Schema({
    id: String,
    img: String,
    alt: String,
    name: String,
    price: String,
    description: String
            
});

var Product = mongoose.model('Product', productSchema);

var product1 = new Product({
    id:"0", 
    img:"tablet", 
    alt:"img-tablet", 
    name:"10-Inch Tablet", 
    price:"269",
    description: "Android 4.3 Jelly Bean, 10.1-inch Full HD (1920 x 1200) Display"
    });

    var product2 = new Product({
        id:"1", 
        img:"shoe", 
        alt:"img-shoe", 
        name:"Running Shoe", 
        price:"48",
        description: "Synthetic and Mesh, Imported, Rubber sole, Flex Film welded upper, HydraMAX moisture-wicking collar lining"
        });

    var product3 = new Product({
        id:"12", 
        img:"watch", 
        alt:"img-watch", 
        name:"Slim Bracelet Watch", 
        price:"48",
        description: "A narrow gold-tone bracelet supports the round case of this  watch, which features three rhinestones marking each hour and a sparkling halo on the bezel"
        });
    
product1.save(function(err, product1) {
      if (err) {return console.error(err);}
    });
product2.save(function(err, product2) {
      if (err) {return console.error(err);}
    });
product3.save(function(err, product3) {
      if (err) {return console.error(err);}
    });

console.log("insert completed");
```

3- Switch to command prompt and run init.js

```shell
node init
```

After completing insert operation, press Ctrl+C to stop process

## 3- Configure Mongoose

1- In the project create a new folder named app_api. This folder will contain all service codes.

2- Create "controllers", "models" and "routes" folders inside the app_api folder.

3- Lets begin by creating Mongoose model file in the app_api/models folder. Create a new file named productModel.js that contains following code snippet:

```javascript
var mongoose = require('mongoose');
var productSchema = mongoose.Schema({
        id: String,
        img: String,
        alt: String,
        name: String,
        price: String,
        description: String
    });

mongoose.model('Product', productSchema);
```

4- Create a new file named db.js in the models folder that contains following code snippet: 

```javascript
var mongoose = require('mongoose');

var db = mongoose.connect("mongodb://localhost/shoppingdb");
var db = mongoose.connection;
db.on('error', console.error);
require('./productModel');
```

## 4- Creating Rest Services

We will develop following rest services during this lab session:  
- Getting all products
  - URL: http://localhost:3000/api/products 
  - Method: GET
  - Path Variable: NA
  - Request Body: NA
  - Response Body: Product list
  - Exceptional Case: The server will return 500 status code when request contains a query parameter named "code".

    Example: http://localhost:3000/api/products?code=111

- Getting a specific product
  - URL: http://localhost:3000/api/products/{id} 
  - Method: GET
  - Path Variable: id of product to be read
  - Request Body: NA
  - Response Body: Product

- Deleting a product
  - URL: http://localhost:3000/api/products/{id} 
  - Method: DELETE
  - Path Variable: id of product to be deleted
  - Request Body: NA
  - Response Body: deleted product

- Saving a product
  - URL: http://localhost:3000/api/products 
  - Method: POST
  - Path Variable: NA
  - Request Body: product data
  - Response Body: saved product

1- Now we should define controller methods. Create a new file named productController.js in the /app_api/controllers folder. Add following code snippet:

```javascript
var mongoose = require('mongoose');
var Product = mongoose.model('Product');

var sendJSONresponse = function(res, status, content) {
  res.status(status);
  res.json(content);
};
```

2- Add the following code snippet for getting all products:

```javascript
module.exports.getProducts = function (req, res) {
  var code = req.query.code;
  
  if(code){
    sendJSONresponse(res, 500, "Internal Server Error");
  }else{
    Product.
        find({}).
        sort({name:"asc"}).
        exec(function(err, data) {
            if (err) {
              return console.error(err);
            };          
            sendJSONresponse(res, 200, data);
          });
        }
  };
```

3- Add the following code snippet for getting a specific product:

```javascript
module.exports.getProductById = function(req, res) {
  var id= req.params.id;
  Product.
    find({id: id}).
    sort({name:"asc"}).
    exec(function(err, data) {
        if (err) {
          return console.error(err);
        };                    
        sendJSONresponse(res, 200, data);
      });
};
```

4- Add the following code snippet for saving new product:

```javascript
module.exports.addProduct = function(req, res, next) {
  var newId=Math.floor(Math.random() * 91 + 10);
  var newProduct= new Product(
  {  "id":newId, 
     "img":req.body.img,
     "alt":req.body.alt,
     "name":req.body.name,
     "price":req.body.price,
     "description":req.body.description });
  
  newProduct.save(function(err){
    if (err) {
            return console.error(err);
    };  
    sendJSONresponse(res, 200, newProduct);
  });   
};
```

5- Add the following code snippet for deleting a product:

```javascript
module.exports.deleteProductById = function(req, res) {
  var id= req.params.id;
  
  Product.
        find({id: id}).
        exec(function(err, data) {
            if (err) {
              return console.error(err);
            };  
            Product.deleteOne({id: id}).
            exec(function(err, deleteResult) {
                if (err) {
                  return console.error(err);
                };                  
                sendJSONresponse(res, 200, data);
              });  
          });
};
```

6- Now, we can continue with defining routes. Create a new file named index.js in the /app_api/routes folder.  Edit file content as shown below:

```javascript
var express = require('express');
var router = express.Router();
var productController = require('../controllers/productController');

router.get('/products', productController.getProducts);
router.get('/products/:id', productController.getProductById);
router.post('/products', productController.addProduct);
router.delete('/products/:id', productController.deleteProductById);
module.exports = router;
```

7- Open app.js and add routes configuration:

```javascript
var routesApi = require('./app_api/routes/index');
….
app.use('/api', routesApi);
```

Delete following lines:

```javascript
var indexRouter = require('./routes/index');
var usersRouter = require('./routes/users');
...
app.use('/', indexRouter);
app.use('/users', usersRouter);
```

8- Add the top of the app.js add following dependency

```javascript
require('./app_api/models/db');
```

9- Start your application

10- Open a rest client application (postman etc.) and type following url: http://localhost:3000/api/products/ change http method as GET. You should get all products as response
![Products](images/rest-1.png)

11- Make GET request to the URL: http://localhost:3000/api/products/1 
You should only see the product with ID value of 1

![Products](images/rest-2.png)

12- Make DELETE request to the URL: http://localhost:3000/api/products/1 
You should get deleted product data

13- Make POST request to the URL: http://localhost:3000/api/products 
You can use following json data as request body

```json
{
"img": "shoe",
"alt": "img-shoe",
"name": "Running Shoe",
"price": "18",
"description": "Synthetic and Mesh, Imported, Rubber sole, Flex Film welded upper, HydraMAX moisture-wicking collar lining"
}
```

Then, you should get a result similar to:

![Products](images/rest-3.png)

## 5- How to enable CORS?

A request for a resource (like an image or a font) outside of the origin is known as a cross-origin request. CORS (cross-origin resource sharing) manages cross-origin requests. Cross-origin resource sharing (CORS) allows AJAX requests to skip the Same-origin policy and access resources from remote hosts. CORS exists for security reasons and to limit which resources a browser can gain access to, from another website. Let’s say our site exists at http://someexampledomain.com and we want the JavaScript files on that site to access http://anotherdomain.com, we can’t do that unless the server at http://anotherdomain.com allows it.

If we try an AJAX/xmlhttprequest to a file like that, we get an error message from the console in our browser.

XMLHttpRequest cannot load http://localhost:3000. No 'Access-Control-Allow-Origin' header is present on the requested resource. Origin 'null' is therefore not allowed access.

To allow exactly that to happend and have our client side code separate from our server and just make it load data, like with frameworks like Angular, Ember, Backbone or the like, we can use the following middleware function in express before we define our routes(app.js):

```javascript
app.use(function(req, res, next) {
  res.header("Access-Control-Allow-Origin", "*");
  res.header("Access-Control-Allow-Headers", "Origin, X-Requested-With, Content-Type, Accept");
  next();
});
```